/* $TOG: XmStrDefs23.ht /main/2 1997/06/18 17:50:15 samborn $ */
#ifndef _XmStrDefs23_h_
#define _XmStrDefs23_h_

#include <X11/Intrinsic.h>

#ifndef _XmConst
#define _XmConst /**/
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* $TOG: makestrs.c /main/11 1998/02/06 11:24:15 kaleb $ */
/* This file is automatically generated. */
/* Default ABI version -- Do not edit */
/* TOG: xmstring.list /main/5 1997/06/20 08:37:02 samborn */

#ifdef XMSTRINGDEFINES
#define XmNdefaultEntryLabelRenderTable "defaultEntryLabelRenderTable"
#define XmNentryLabelRenderTable "entryLabelRenderTable"
#define XmNpixmapPlacement "pixmapPlacement"
#define XmCPixmapPlacement "PixmapPlacement"
#define XmRPixmapPlacement "PixmapPlacement"
#define XmNpixmapTextPadding "pixmapTextPadding"
#define XmNfontStyle "fontStyle"
#define XmCFontStyle "FontStyle"
#define XmNfontSize "fontSize"
#define XmCFontSize "FontSize"
#define XmNfontFoundry "fontFoundry"
#define XmCFontFoundry "FontFoundry"
#define XmNfontEncoding "fontEncoding"
#define XmCFontEncoding "FontEncoding"
#define XmNxftFont "xftFont"
#define XmCXftFont "XftFont"
#define XmSUTF8_STRING "UTF8_STRING"
#else
externalref _XmConst char _XmStrings23[];
#ifndef XmNdefaultEntryLabelRenderTable
#define XmNdefaultEntryLabelRenderTable ((char*)&_XmStrings23[0])
#endif
#ifndef XmNentryLabelRenderTable
#define XmNentryLabelRenderTable ((char*)&_XmStrings23[29])
#endif
#ifndef XmNpixmapPlacement
#define XmNpixmapPlacement ((char*)&_XmStrings23[51])
#endif
#ifndef XmCPixmapPlacement
#define XmCPixmapPlacement ((char*)&_XmStrings23[67])
#endif
#ifndef XmRPixmapPlacement
#define XmRPixmapPlacement ((char*)&_XmStrings23[83])
#endif
#ifndef XmNpixmapTextPadding
#define XmNpixmapTextPadding ((char*)&_XmStrings23[99])
#endif
#ifndef XmNfontStyle
#define XmNfontStyle ((char*)&_XmStrings23[117])
#endif
#ifndef XmCFontStyle
#define XmCFontStyle ((char*)&_XmStrings23[127])
#endif
#ifndef XmNfontSize
#define XmNfontSize ((char*)&_XmStrings23[137])
#endif
#ifndef XmCFontSize
#define XmCFontSize ((char*)&_XmStrings23[146])
#endif
#ifndef XmNfontFoundry
#define XmNfontFoundry ((char*)&_XmStrings23[155])
#endif
#ifndef XmCFontFoundry
#define XmCFontFoundry ((char*)&_XmStrings23[167])
#endif
#ifndef XmNfontEncoding
#define XmNfontEncoding ((char*)&_XmStrings23[179])
#endif
#ifndef XmCFontEncoding
#define XmCFontEncoding ((char*)&_XmStrings23[192])
#endif
#ifndef XmNxftFont
#define XmNxftFont ((char*)&_XmStrings23[205])
#endif
#ifndef XmCXftFont
#define XmCXftFont ((char*)&_XmStrings23[213])
#endif
#ifndef XmSUTF8_STRING
#define XmSUTF8_STRING ((char*)&_XmStrings23[221])
#endif
#endif /* XMSTRINGDEFINES */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* _XmStrDefs23_h_ */
