/*
 * WARNING: This file is overwritten at code generation time.
 * Any changes to this file will be lost.
 */
/*
 *
 * This constant file generated by Builder Xcessory 3.5.1.
 *
 */
#ifndef creation_H
#define creation_H
#define allviewpix	allviewpix_icon
static char * allviewpix_icon[] = {
"24 24 3 1",
". c #c0cc0cc0c",
"X c #000",
"o c #6363b8b8ffff",
".....XXXXX....XXXXX.....",
"...XXoooooXXXXoooooXX...",
"..XooooooooXXooooooooX..",
".XoXXXooooXooXooooXXooX.",
".XoXooXoooXooXoooXooXoX.",
"XooXooXooXooooXooXoooooX",
"XooXXXoooXooooXoooXXoooX",
"XooXoooooXooooXoooooXooX",
"XooXoooooXooooXooXooXooX",
"XooXoooooXXXXXXoooXXoooX",
".XooooooXXXooXoXXoooooX.",
".XoooooXooXooXoooXooooX.",
"..XoooXooooXXoooooXooX..",
"...XXoXoooXXXXooooXXX...",
".....XXXXXooooXXXXXX....",
".....XoooooooooooooX....",
".....XoooXoooooXoooX....",
".....XoooXXoooXXoooX....",
".....XoooXoXoXoXoooX....",
"......XooXooXooXooX.....",
"......XooXoooooXooX.....",
".......XoooooooooX......",
"........XXoooooXX.......",
"..........XXXXX........."
};


extern Widget Createform(Widget parent);
#endif
