/* $TOG: XmStrDefs22.ht /main/2 1997/06/18 17:50:15 samborn $ */
#ifndef _XmStrDefs22_h_
#define _XmStrDefs22_h_

#include <X11/Intrinsic.h>

#ifndef _XmConst
#define _XmConst /**/
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* $TOG: makestrs.c /main/11 1998/02/06 11:24:15 kaleb $ */
/* This file is automatically generated. */
/* Default ABI version -- Do not edit */
/* TOG: xmstring.list /main/5 1997/06/20 08:37:02 samborn */

#ifdef XMSTRINGDEFINES
#define XmNitemFoundCallback "itemFoundCallback"
#define XmNitemNotFoundCallback "itemNotFoundCallback"
#define XmRXmTabSide "XmTabSide"
#define XmRTabSide "TabSide"
#define XmRXmPixmapPlacement "XmPixmapPlacement"
#define XmRXmPixmap "XmPixmap"
#define XmRXmPixel "XmPixel"
#define XmRXmIconPlacement "XmIconPlacement"
#define XmRXmHierarchyNodeState "XmHierarchyNodeState"
#define XmRXmFillOption "XmFillOption"
#define XmRXmConnectStyle "XmConnectStyle"
#define XmRXmColorMode "XmColorMode"
#define XmRXmAlignment "XmAlignment"
#define XmRTabStyle "TabStyle"
#define XmRTabOrientation "TabOrientation"
#define XmRTabMode "TabMode"
#define XmRTabList "TabList"
#define XmRTabEdge "TabEdge"
#define XmRTabArrowPlacement "TabArrowPlacement"
#define XmNxlfdString "xlfdString"
#define XmNverticalNodeSpace "verticalNodeSpace"
#define XmNverticalMargin "verticalMargin"
#define XmNverifyTextFailedCallback "verifyTextFailedCallback"
#define XmNverifyTextCallback "verifyTextCallback"
#define XmNverify "verify"
#define XmNvalues "values"
#define XmNuseTextField "useTextField"
#define XmNuseScaling "useScaling"
#define XmNuseImageCache "useImageCache"
#define XmNupdateTextCallback "updateTextCallback"
#define XmNupdateShellCallback "updateShellCallback"
#define XmNunselectCallback "unselectCallback"
#define XmNuniformTabSize "uniformTabSize"
#define XmNtraversalIndex "traversalIndex"
#define XmNtitleString "titleString"
#define XmNtextString "textString"
#define XmNtextRows "textRows"
#define XmNtearOffLabelString "tearOffLabelString"
#define XmNtabTearOffEnabled "tabTearOffEnabled"
#define XmNtabStyle "tabStyle"
#define XmNtabStringDirection "tabStringDirection"
#define XmNtabSide "tabSide"
#define XmNtabSelectedCallback "tabSelectedCallback"
#define XmNtabSelectPixmap "tabSelectPixmap"
#define XmNtabSelectColor "tabSelectColor"
#define XmNtabPixmapPlacement "tabPixmapPlacement"
#define XmNtabOrientation "tabOrientation"
#define XmNtabOffset "tabOffset"
#define XmNtabMode "tabMode"
#define XmNtabMarginWidth "tabMarginWidth"
#define XmNtabMarginHeight "tabMarginHeight"
#define XmNtabList "tabList"
#define XmNtabLabelString "tabLabelString"
#define XmNtabLabelSpacing "tabLabelSpacing"
#define XmNtabLabelPixmap "tabLabelPixmap"
#define XmNtabForeground "tabForeground"
#define XmNtabEdge "tabEdge"
#define XmNtabCornerPercent "tabCornerPercent"
#define XmNtabBoxWidget "tabBoxWidget"
#define XmNtabBackgroundPixmap "tabBackgroundPixmap"
#define XmNtabBackground "tabBackground"
#define XmNtabAutoSelect "tabAutoSelect"
#define XmNtabArrowPlacement "tabArrowPlacement"
#define XmNtabAlignment "tabAlignment"
#define XmNstackedEffect "stackedEffect"
#define XmNsortFunctions "sortFunctions"
#define XmNsliderTogLabel "sliderTogLabel"
#define XmNsizeString "sizeString"
#define XmNshowValue "showValue"
#define XmNshowSash "showSash"
#define XmNshowNameString "showNameString"
#define XmNshowLabel "showLabel"
#define XmNshowFontName "showFontName"
#define XmNshowFind "showFind"
#define XmNselectedIndex "selectedIndex"
#define XmNselectedColumn "selectedColumn"
#define XmNselectCallback "selectCallback"
#define XmNscalingString "scalingString"
#define XmNsashTranslations "sashTranslations"
#define XmNsampleText "sampleText"
#define XmNrgbFile "rgbFile"
#define XmNresizeToPreferred "resizeToPreferred"
#define XmNresizeCallback "resizeCallback"
#define XmNredSliderLabel "redSliderLabel"
#define XmNpropSpaceString "propSpaceString"
#define XmNpreferredPaneSize "preferredPaneSize"
#define XmNposition "position"
#define XmNpopupShellWidget "popupShellWidget"
#define XmNpopupOffset "popupOffset"
#define XmNpopupCursor "popupCursor"
#define XmNpixmapWidth "pixmapWidth"
#define XmNpixmapHeight "pixmapHeight"
#define XmNparentNode "parentNode"
#define XmNotherString "otherString"
#define XmNoptionString "optionString"
#define XmNopenFolderPixmap "openFolderPixmap"
#define XmNopenClosePadding "openClosePadding"
#define XmNnumValues "numValues"
#define XmNnumStacks "numStacks"
#define XmNnumRows "numRows"
#define XmNnodeStateChangedCallback "nodeStateChangedCallback"
#define XmNnodeStateCallback "nodeStateCallback"
#define XmNnodeStateBegEndCallback "nodeStateBegEndCallback"
#define XmNnodeState "nodeState"
#define XmNnoCellError "noCellError"
#define XmNmonoSpaceString "monoSpaceString"
#define XmNminimumVerticalCells "minimumVerticalCells"
#define XmNminimumHorizontalCells "minimumHorizontalCells"
#define XmNminimumCellWidth "minimumCellWidth"
#define XmNminimumCellHeight "minimumCellHeight"
#define XmNmargin "margin"
#define XmNlist "list"
#define XmNlineWidth "lineWidth"
#define XmNlineColor "lineColor"
#define XmNlabel "label"
#define XmNitalicString "italicString"
#define XmNinsertBefore "insertBefore"
#define XmNindentSpace "indentSpace"
#define XmNiconTextPadding "iconTextPadding"
#define XmNiconPlacement "iconPlacement"
#define XmNhorizontalNodeSpace "horizontalNodeSpace"
#define XmNhorizontalMargin "horizontalMargin"
#define XmNgreenSliderLabel "greenSliderLabel"
#define XmNfreeTabPixmap "freeTabPixmap"
#define XmNfirstRow "firstRow"
#define XmNfirstColumnPixmaps "firstColumnPixmaps"
#define XmNfirstColumn "firstColumn"
#define XmNfindLabel "findLabel"
#define XmNfillOption "fillOption"
#define XmNfileReadError "fileReadError"
#define XmNfamilyString "familyString"
#define XmNequalSize "equalSize"
#define XmNentryLabelString "entryLabelString"
#define XmNentryData "entryData"
#define XmNencodingString "encodingString"
#define XmNencodingList "encodingList"
#define XmNdoubleClickCallback "doubleClickCallback"
#define XmNdefaultEncodingString "defaultEncodingString"
#define XmNdFieldPrefWidth "dFieldPrefWidth"
#define XmNdFieldPrefHeight "dFieldPrefHeight"
#define XmNdFieldMinWidth "dFieldMinWidth"
#define XmNdFieldMinHeight "dFieldMinHeight"
#define XmNdFieldMaxWidth "dFieldMaxWidth"
#define XmNdFieldMaxHeight "dFieldMaxHeight"
#define XmNcustomizedCombinationBox "customizedCombinationBox"
#define XmNcursor "cursor"
#define XmNcurrentFont "currentFont"
#define XmNconnectStyle "connectStyle"
#define XmNcomboTranslations "comboTranslations"
#define XmNcolumnTitles "columnTitles"
#define XmNcolorName "colorName"
#define XmNcolorMode "colorMode"
#define XmNcolorListTogLabel "colorListTogLabel"
#define XmNcellY "cellY"
#define XmNcellX "cellX"
#define XmNbothString "bothString"
#define XmNboldString "boldString"
#define XmNblueSliderLabel "blueSliderLabel"
#define XmCDistribution "Distribution"
#define XmCFillStyle "FillStyle"
#define XmCItemSpacing "ItemSpacing"
#define XmCLabelSpacing "LabelSpacing"
#define XmCShowLabel "ShowLabel"
#define XmCStretchable "Stretchable"
#define XmNdefaultEntryLabelAlignment "defaultEntryLabelAlignment"
#define XmNdefaultEntryLabelFontList "defaultEntryLabelFontList"
#define XmNdefaultFillStyle "defaultFillStyle"
#define XmNdistribution "distribution"
#define XmNentryLabelAlignment "entryLabelAlignment"
#define XmNentryLabelFontList "entryLabelFontList"
#define XmNentryLabelPixmap "entryLabelPixmap"
#define XmNentryLabelType "entryLabelType"
#define XmNfillStyle "fillStyle"
#define XmNitemSpacing "itemSpacing"
#define XmNlabelSpacing "labelSpacing"
#define XmNshowEntryLabel "showEntryLabel"
#define XmNstretchable "stretchable"
#define XmRDistribution "Distribution"
#define XmRFillStyle "FillStyle"
#define XmCEqualSize "EqualSize"
#define XmCFillOption "FillOption"
#define XmCColorMode "ColorMode"
#define XmCFileReadError "FileReadError"
#define XmCNoCellError "NoCellError"
#define XmCSliderLabel "SliderLabel"
#define XmCTogLabel "TogLabel"
#define XmCAutoFill "AutoFill"
#define XmCPicture "Picture"
#define XmNautoFill "autoFill"
#define XmNpicture "picture"
#define XmNpictureErrorCallback "pictureErrorCallback"
#define XmNvalidateCallback "validateCallback"
#define XmCFindLabel "FindLabel"
#define XmCShowFind "ShowFind"
#define XmNentryBackground "entryBackground"
#define XmCColumnTitles "ColumnTitles"
#define XmCEntryData "EntryData"
#define XmCFirstColumnPixmaps "FirstColumnPixmaps"
#define XmCFirstLocation "FirstLocation"
#define XmCNewVisualStyle "NewVisualStyle"
#define XmCNumRows "NumRows"
#define XmCScrollBar "ScrollBar"
#define XmCSelectedColumn "SelectedColumn"
#define XmNnewVisualStyle "newVisualStyle"
#define XmCPreferredPaneSize "PreferredPaneSize"
#define XmCShowSash "ShowSash"
#define XmC100DPIString "100DPIString"
#define XmC75DPIString "75DPIString"
#define XmCAnyLowerString "AnyLowerString"
#define XmCAnyString "AnyString"
#define XmCBoldString "BoldString"
#define XmCBothString "BothString"
#define XmCDefaultEncodingString "DefaultEncodingString"
#define XmCEncodingList "EncodingList"
#define XmCEncodingString "EncodingString"
#define XmCFamilyString "FamilyString"
#define XmCItalicString "ItalicString"
#define XmCMonoSpaceString "MonoSpaceString"
#define XmCOptionString "OptionString"
#define XmCOtherString "OtherString"
#define XmCPropSpaceString "PropSpaceString"
#define XmCSampleText "SampleText"
#define XmCScalingString "ScalingString"
#define XmCShowNameString "ShowNameString"
#define XmCSizeString "SizeString"
#define XmCTextRows "TextRows"
#define XmCXlfdSpaceString "XlfdSpaceString"
#define XmN100DPIstring "100DPIstring"
#define XmN75DPIstring "75DPIstring"
#define XmNanyLowerString "anyLowerString"
#define XmNanyString "anyString"
#define XmCCellX "CellX"
#define XmCCellY "CellY"
#define XmCDefaultCells "DefaultCells"
#define XmCMinimumCellSize "MinimumCellSize"
#define XmCIconPlacement "IconPlacement"
#define XmNpixmapDepth "pixmapDepth"
#define XmCAutoClose "AutoClose"
#define XmCInsertBefore "InsertBefore"
#define XmCNodeState "NodeState"
#define XmCNodeStateCallback "NodeStateCallback"
#define XmCNodeStateChangedCallback "NodeStateChangedCallback"
#define XmCParentNode "ParentNode"
#define XmNautoClose "autoClose"
#define XmNcloseFolderPixmap "closeFolderPixmap"
#define XmNnodeCloseFolderPixmap "nodeCloseFolderPixmap"
#define XmNnodeOpenFolderPixmap "nodeOpenFolderPixmap"
#define XmNnodeStateBeginEndCallback "nodeStateBeginEndCallback"
#define XmCConstrainWidth "ConstrainWidth"
#define XmCIndentSpace "IndentSpace"
#define XmNconnectNodes "connectNodes"
#define XmNconstrainWidth "constrainWidth"
#define XmCNumStacks "NumStacks"
#define XmCSelectedIndex "SelectedIndex"
#define XmCStackedEffect "StackedEffect"
#define XmCTabAutoSelect "TabAutoSelect"
#define XmCTabCornerPercent "TabCornerPercent"
#define XmCTabEdge "TabEdge"
#define XmCTabLabelSpacing "TabLabelSpacing"
#define XmCTabMode "TabMode"
#define XmCTabOffset "TabOffset"
#define XmCTabOrientation "TabOrientation"
#define XmCTabSelectColor "TabSelectColor"
#define XmCTabSelectPixmap "TabSelectPixmap"
#define XmCTabStyle "TabStyle"
#define XmCTraversalIndex "TraversalIndex"
#define XmCUniformTabSize "UniformTabSize"
#define XmCUseImageCache "UseImageCache"
#define XmCFreeTabPixmap "FreeTabPixmap"
#define XmCICSEnhancementPakResourceError "ICSEnhancementPakResourceError"
#define XmCTabLabelPixmap "TabLabelPixmap"
#define XmCTabLabelString "TabLabelString"
#define XmCTabPixmapPlacement "TabPixmapPlacement"
#define XmCTabSide "TabSide"
#define XmNillegalResourceValue "illegalResourceValue"
#define XmCCompressStyle "CompressStyle"
#define XmCConnectStyle "ConnectStyle"
#define XmCHorizontalDelta "HorizontalDelta"
#define XmCLineWidth "LineWidth"
#define XmCOpenClosePadding "OpenClosePadding"
#define XmCVerticalDelta "VerticalDelta"
#define XmNcompressStyle "compressStyle"
#define XmNhorizontalDelta "horizontalDelta"
#define XmNlineBackgroundColor "lineBackgroundColor"
#define XmNlineStyle "lineStyle"
#define XmNverticalDelta "verticalDelta"
#define XmRXmCompressStyle "XmCompressStyle"
#define XmRXmLineStyle "XmLineStyle"
#define XmCPopupOffset "PopupOffset"
#define XmCUseTextField "UseTextField"
#define XmCVerify "Verify"
#else
externalref _XmConst char _XmStrings22[];
#ifndef XmNitemFoundCallback
#define XmNitemFoundCallback ((char*)&_XmStrings22[0])
#endif
#ifndef XmNitemNotFoundCallback
#define XmNitemNotFoundCallback ((char*)&_XmStrings22[18])
#endif
#ifndef XmRXmTabSide
#define XmRXmTabSide ((char*)&_XmStrings22[39])
#endif
#ifndef XmRTabSide
#define XmRTabSide ((char*)&_XmStrings22[49])
#endif
#ifndef XmRXmPixmapPlacement
#define XmRXmPixmapPlacement ((char*)&_XmStrings22[57])
#endif
#ifndef XmRXmPixmap
#define XmRXmPixmap ((char*)&_XmStrings22[75])
#endif
#ifndef XmRXmPixel
#define XmRXmPixel ((char*)&_XmStrings22[84])
#endif
#ifndef XmRXmIconPlacement
#define XmRXmIconPlacement ((char*)&_XmStrings22[92])
#endif
#ifndef XmRXmHierarchyNodeState
#define XmRXmHierarchyNodeState ((char*)&_XmStrings22[108])
#endif
#ifndef XmRXmFillOption
#define XmRXmFillOption ((char*)&_XmStrings22[129])
#endif
#ifndef XmRXmConnectStyle
#define XmRXmConnectStyle ((char*)&_XmStrings22[142])
#endif
#ifndef XmRXmColorMode
#define XmRXmColorMode ((char*)&_XmStrings22[157])
#endif
#ifndef XmRXmAlignment
#define XmRXmAlignment ((char*)&_XmStrings22[169])
#endif
#ifndef XmRTabStyle
#define XmRTabStyle ((char*)&_XmStrings22[181])
#endif
#ifndef XmRTabOrientation
#define XmRTabOrientation ((char*)&_XmStrings22[190])
#endif
#ifndef XmRTabMode
#define XmRTabMode ((char*)&_XmStrings22[205])
#endif
#ifndef XmRTabList
#define XmRTabList ((char*)&_XmStrings22[213])
#endif
#ifndef XmRTabEdge
#define XmRTabEdge ((char*)&_XmStrings22[221])
#endif
#ifndef XmRTabArrowPlacement
#define XmRTabArrowPlacement ((char*)&_XmStrings22[229])
#endif
#ifndef XmNxlfdString
#define XmNxlfdString ((char*)&_XmStrings22[247])
#endif
#ifndef XmNverticalNodeSpace
#define XmNverticalNodeSpace ((char*)&_XmStrings22[258])
#endif
#ifndef XmNverticalMargin
#define XmNverticalMargin ((char*)&_XmStrings22[276])
#endif
#ifndef XmNverifyTextFailedCallback
#define XmNverifyTextFailedCallback ((char*)&_XmStrings22[291])
#endif
#ifndef XmNverifyTextCallback
#define XmNverifyTextCallback ((char*)&_XmStrings22[316])
#endif
#ifndef XmNverify
#define XmNverify ((char*)&_XmStrings22[335])
#endif
#ifndef XmNvalues
#define XmNvalues ((char*)&_XmStrings22[342])
#endif
#ifndef XmNuseTextField
#define XmNuseTextField ((char*)&_XmStrings22[349])
#endif
#ifndef XmNuseScaling
#define XmNuseScaling ((char*)&_XmStrings22[362])
#endif
#ifndef XmNuseImageCache
#define XmNuseImageCache ((char*)&_XmStrings22[373])
#endif
#ifndef XmNupdateTextCallback
#define XmNupdateTextCallback ((char*)&_XmStrings22[387])
#endif
#ifndef XmNupdateShellCallback
#define XmNupdateShellCallback ((char*)&_XmStrings22[406])
#endif
#ifndef XmNunselectCallback
#define XmNunselectCallback ((char*)&_XmStrings22[426])
#endif
#ifndef XmNuniformTabSize
#define XmNuniformTabSize ((char*)&_XmStrings22[443])
#endif
#ifndef XmNtraversalIndex
#define XmNtraversalIndex ((char*)&_XmStrings22[458])
#endif
#ifndef XmNtitleString
#define XmNtitleString ((char*)&_XmStrings22[473])
#endif
#ifndef XmNtextString
#define XmNtextString ((char*)&_XmStrings22[485])
#endif
#ifndef XmNtextRows
#define XmNtextRows ((char*)&_XmStrings22[496])
#endif
#ifndef XmNtearOffLabelString
#define XmNtearOffLabelString ((char*)&_XmStrings22[505])
#endif
#ifndef XmNtabTearOffEnabled
#define XmNtabTearOffEnabled ((char*)&_XmStrings22[524])
#endif
#ifndef XmNtabStyle
#define XmNtabStyle ((char*)&_XmStrings22[542])
#endif
#ifndef XmNtabStringDirection
#define XmNtabStringDirection ((char*)&_XmStrings22[551])
#endif
#ifndef XmNtabSide
#define XmNtabSide ((char*)&_XmStrings22[570])
#endif
#ifndef XmNtabSelectedCallback
#define XmNtabSelectedCallback ((char*)&_XmStrings22[578])
#endif
#ifndef XmNtabSelectPixmap
#define XmNtabSelectPixmap ((char*)&_XmStrings22[598])
#endif
#ifndef XmNtabSelectColor
#define XmNtabSelectColor ((char*)&_XmStrings22[614])
#endif
#ifndef XmNtabPixmapPlacement
#define XmNtabPixmapPlacement ((char*)&_XmStrings22[629])
#endif
#ifndef XmNtabOrientation
#define XmNtabOrientation ((char*)&_XmStrings22[648])
#endif
#ifndef XmNtabOffset
#define XmNtabOffset ((char*)&_XmStrings22[663])
#endif
#ifndef XmNtabMode
#define XmNtabMode ((char*)&_XmStrings22[673])
#endif
#ifndef XmNtabMarginWidth
#define XmNtabMarginWidth ((char*)&_XmStrings22[681])
#endif
#ifndef XmNtabMarginHeight
#define XmNtabMarginHeight ((char*)&_XmStrings22[696])
#endif
#ifndef XmNtabList
#define XmNtabList ((char*)&_XmStrings22[712])
#endif
#ifndef XmNtabLabelString
#define XmNtabLabelString ((char*)&_XmStrings22[720])
#endif
#ifndef XmNtabLabelSpacing
#define XmNtabLabelSpacing ((char*)&_XmStrings22[735])
#endif
#ifndef XmNtabLabelPixmap
#define XmNtabLabelPixmap ((char*)&_XmStrings22[751])
#endif
#ifndef XmNtabForeground
#define XmNtabForeground ((char*)&_XmStrings22[766])
#endif
#ifndef XmNtabEdge
#define XmNtabEdge ((char*)&_XmStrings22[780])
#endif
#ifndef XmNtabCornerPercent
#define XmNtabCornerPercent ((char*)&_XmStrings22[788])
#endif
#ifndef XmNtabBoxWidget
#define XmNtabBoxWidget ((char*)&_XmStrings22[805])
#endif
#ifndef XmNtabBackgroundPixmap
#define XmNtabBackgroundPixmap ((char*)&_XmStrings22[818])
#endif
#ifndef XmNtabBackground
#define XmNtabBackground ((char*)&_XmStrings22[838])
#endif
#ifndef XmNtabAutoSelect
#define XmNtabAutoSelect ((char*)&_XmStrings22[852])
#endif
#ifndef XmNtabArrowPlacement
#define XmNtabArrowPlacement ((char*)&_XmStrings22[866])
#endif
#ifndef XmNtabAlignment
#define XmNtabAlignment ((char*)&_XmStrings22[884])
#endif
#ifndef XmNstackedEffect
#define XmNstackedEffect ((char*)&_XmStrings22[897])
#endif
#ifndef XmNsortFunctions
#define XmNsortFunctions ((char*)&_XmStrings22[911])
#endif
#ifndef XmNsliderTogLabel
#define XmNsliderTogLabel ((char*)&_XmStrings22[925])
#endif
#ifndef XmNsizeString
#define XmNsizeString ((char*)&_XmStrings22[940])
#endif
#ifndef XmNshowValue
#define XmNshowValue ((char*)&_XmStrings22[951])
#endif
#ifndef XmNshowSash
#define XmNshowSash ((char*)&_XmStrings22[961])
#endif
#ifndef XmNshowNameString
#define XmNshowNameString ((char*)&_XmStrings22[970])
#endif
#ifndef XmNshowLabel
#define XmNshowLabel ((char*)&_XmStrings22[985])
#endif
#ifndef XmNshowFontName
#define XmNshowFontName ((char*)&_XmStrings22[995])
#endif
#ifndef XmNshowFind
#define XmNshowFind ((char*)&_XmStrings22[1008])
#endif
#ifndef XmNselectedIndex
#define XmNselectedIndex ((char*)&_XmStrings22[1017])
#endif
#ifndef XmNselectedColumn
#define XmNselectedColumn ((char*)&_XmStrings22[1031])
#endif
#ifndef XmNselectCallback
#define XmNselectCallback ((char*)&_XmStrings22[1046])
#endif
#ifndef XmNscalingString
#define XmNscalingString ((char*)&_XmStrings22[1061])
#endif
#ifndef XmNsashTranslations
#define XmNsashTranslations ((char*)&_XmStrings22[1075])
#endif
#ifndef XmNsampleText
#define XmNsampleText ((char*)&_XmStrings22[1092])
#endif
#ifndef XmNrgbFile
#define XmNrgbFile ((char*)&_XmStrings22[1103])
#endif
#ifndef XmNresizeToPreferred
#define XmNresizeToPreferred ((char*)&_XmStrings22[1111])
#endif
#ifndef XmNresizeCallback
#define XmNresizeCallback ((char*)&_XmStrings22[1129])
#endif
#ifndef XmNredSliderLabel
#define XmNredSliderLabel ((char*)&_XmStrings22[1144])
#endif
#ifndef XmNpropSpaceString
#define XmNpropSpaceString ((char*)&_XmStrings22[1159])
#endif
#ifndef XmNpreferredPaneSize
#define XmNpreferredPaneSize ((char*)&_XmStrings22[1175])
#endif
#ifndef XmNposition
#define XmNposition ((char*)&_XmStrings22[1193])
#endif
#ifndef XmNpopupShellWidget
#define XmNpopupShellWidget ((char*)&_XmStrings22[1202])
#endif
#ifndef XmNpopupOffset
#define XmNpopupOffset ((char*)&_XmStrings22[1219])
#endif
#ifndef XmNpopupCursor
#define XmNpopupCursor ((char*)&_XmStrings22[1231])
#endif
#ifndef XmNpixmapWidth
#define XmNpixmapWidth ((char*)&_XmStrings22[1243])
#endif
#ifndef XmNpixmapHeight
#define XmNpixmapHeight ((char*)&_XmStrings22[1255])
#endif
#ifndef XmNparentNode
#define XmNparentNode ((char*)&_XmStrings22[1268])
#endif
#ifndef XmNotherString
#define XmNotherString ((char*)&_XmStrings22[1279])
#endif
#ifndef XmNoptionString
#define XmNoptionString ((char*)&_XmStrings22[1291])
#endif
#ifndef XmNopenFolderPixmap
#define XmNopenFolderPixmap ((char*)&_XmStrings22[1304])
#endif
#ifndef XmNopenClosePadding
#define XmNopenClosePadding ((char*)&_XmStrings22[1321])
#endif
#ifndef XmNnumValues
#define XmNnumValues ((char*)&_XmStrings22[1338])
#endif
#ifndef XmNnumStacks
#define XmNnumStacks ((char*)&_XmStrings22[1348])
#endif
#ifndef XmNnumRows
#define XmNnumRows ((char*)&_XmStrings22[1358])
#endif
#ifndef XmNnodeStateChangedCallback
#define XmNnodeStateChangedCallback ((char*)&_XmStrings22[1366])
#endif
#ifndef XmNnodeStateCallback
#define XmNnodeStateCallback ((char*)&_XmStrings22[1391])
#endif
#ifndef XmNnodeStateBegEndCallback
#define XmNnodeStateBegEndCallback ((char*)&_XmStrings22[1409])
#endif
#ifndef XmNnodeState
#define XmNnodeState ((char*)&_XmStrings22[1433])
#endif
#ifndef XmNnoCellError
#define XmNnoCellError ((char*)&_XmStrings22[1443])
#endif
#ifndef XmNmonoSpaceString
#define XmNmonoSpaceString ((char*)&_XmStrings22[1455])
#endif
#ifndef XmNminimumVerticalCells
#define XmNminimumVerticalCells ((char*)&_XmStrings22[1471])
#endif
#ifndef XmNminimumHorizontalCells
#define XmNminimumHorizontalCells ((char*)&_XmStrings22[1492])
#endif
#ifndef XmNminimumCellWidth
#define XmNminimumCellWidth ((char*)&_XmStrings22[1515])
#endif
#ifndef XmNminimumCellHeight
#define XmNminimumCellHeight ((char*)&_XmStrings22[1532])
#endif
#ifndef XmNmargin
#define XmNmargin ((char*)&_XmStrings22[1550])
#endif
#ifndef XmNlist
#define XmNlist ((char*)&_XmStrings22[1557])
#endif
#ifndef XmNlineWidth
#define XmNlineWidth ((char*)&_XmStrings22[1562])
#endif
#ifndef XmNlineColor
#define XmNlineColor ((char*)&_XmStrings22[1572])
#endif
#ifndef XmNlabel
#define XmNlabel ((char*)&_XmStrings22[1582])
#endif
#ifndef XmNitalicString
#define XmNitalicString ((char*)&_XmStrings22[1588])
#endif
#ifndef XmNinsertBefore
#define XmNinsertBefore ((char*)&_XmStrings22[1601])
#endif
#ifndef XmNindentSpace
#define XmNindentSpace ((char*)&_XmStrings22[1614])
#endif
#ifndef XmNiconTextPadding
#define XmNiconTextPadding ((char*)&_XmStrings22[1626])
#endif
#ifndef XmNiconPlacement
#define XmNiconPlacement ((char*)&_XmStrings22[1642])
#endif
#ifndef XmNhorizontalNodeSpace
#define XmNhorizontalNodeSpace ((char*)&_XmStrings22[1656])
#endif
#ifndef XmNhorizontalMargin
#define XmNhorizontalMargin ((char*)&_XmStrings22[1676])
#endif
#ifndef XmNgreenSliderLabel
#define XmNgreenSliderLabel ((char*)&_XmStrings22[1693])
#endif
#ifndef XmNfreeTabPixmap
#define XmNfreeTabPixmap ((char*)&_XmStrings22[1710])
#endif
#ifndef XmNfirstRow
#define XmNfirstRow ((char*)&_XmStrings22[1724])
#endif
#ifndef XmNfirstColumnPixmaps
#define XmNfirstColumnPixmaps ((char*)&_XmStrings22[1733])
#endif
#ifndef XmNfirstColumn
#define XmNfirstColumn ((char*)&_XmStrings22[1752])
#endif
#ifndef XmNfindLabel
#define XmNfindLabel ((char*)&_XmStrings22[1764])
#endif
#ifndef XmNfillOption
#define XmNfillOption ((char*)&_XmStrings22[1774])
#endif
#ifndef XmNfileReadError
#define XmNfileReadError ((char*)&_XmStrings22[1785])
#endif
#ifndef XmNfamilyString
#define XmNfamilyString ((char*)&_XmStrings22[1799])
#endif
#ifndef XmNequalSize
#define XmNequalSize ((char*)&_XmStrings22[1812])
#endif
#ifndef XmNentryLabelString
#define XmNentryLabelString ((char*)&_XmStrings22[1822])
#endif
#ifndef XmNentryData
#define XmNentryData ((char*)&_XmStrings22[1839])
#endif
#ifndef XmNencodingString
#define XmNencodingString ((char*)&_XmStrings22[1849])
#endif
#ifndef XmNencodingList
#define XmNencodingList ((char*)&_XmStrings22[1864])
#endif
#ifndef XmNdoubleClickCallback
#define XmNdoubleClickCallback ((char*)&_XmStrings22[1877])
#endif
#ifndef XmNdefaultEncodingString
#define XmNdefaultEncodingString ((char*)&_XmStrings22[1897])
#endif
#ifndef XmNdFieldPrefWidth
#define XmNdFieldPrefWidth ((char*)&_XmStrings22[1919])
#endif
#ifndef XmNdFieldPrefHeight
#define XmNdFieldPrefHeight ((char*)&_XmStrings22[1935])
#endif
#ifndef XmNdFieldMinWidth
#define XmNdFieldMinWidth ((char*)&_XmStrings22[1952])
#endif
#ifndef XmNdFieldMinHeight
#define XmNdFieldMinHeight ((char*)&_XmStrings22[1967])
#endif
#ifndef XmNdFieldMaxWidth
#define XmNdFieldMaxWidth ((char*)&_XmStrings22[1983])
#endif
#ifndef XmNdFieldMaxHeight
#define XmNdFieldMaxHeight ((char*)&_XmStrings22[1998])
#endif
#ifndef XmNcustomizedCombinationBox
#define XmNcustomizedCombinationBox ((char*)&_XmStrings22[2014])
#endif
#ifndef XmNcursor
#define XmNcursor ((char*)&_XmStrings22[2039])
#endif
#ifndef XmNcurrentFont
#define XmNcurrentFont ((char*)&_XmStrings22[2046])
#endif
#ifndef XmNconnectStyle
#define XmNconnectStyle ((char*)&_XmStrings22[2058])
#endif
#ifndef XmNcomboTranslations
#define XmNcomboTranslations ((char*)&_XmStrings22[2071])
#endif
#ifndef XmNcolumnTitles
#define XmNcolumnTitles ((char*)&_XmStrings22[2089])
#endif
#ifndef XmNcolorName
#define XmNcolorName ((char*)&_XmStrings22[2102])
#endif
#ifndef XmNcolorMode
#define XmNcolorMode ((char*)&_XmStrings22[2112])
#endif
#ifndef XmNcolorListTogLabel
#define XmNcolorListTogLabel ((char*)&_XmStrings22[2122])
#endif
#ifndef XmNcellY
#define XmNcellY ((char*)&_XmStrings22[2140])
#endif
#ifndef XmNcellX
#define XmNcellX ((char*)&_XmStrings22[2146])
#endif
#ifndef XmNbothString
#define XmNbothString ((char*)&_XmStrings22[2152])
#endif
#ifndef XmNboldString
#define XmNboldString ((char*)&_XmStrings22[2163])
#endif
#ifndef XmNblueSliderLabel
#define XmNblueSliderLabel ((char*)&_XmStrings22[2174])
#endif
#ifndef XmCDistribution
#define XmCDistribution ((char*)&_XmStrings22[2190])
#endif
#ifndef XmCFillStyle
#define XmCFillStyle ((char*)&_XmStrings22[2203])
#endif
#ifndef XmCItemSpacing
#define XmCItemSpacing ((char*)&_XmStrings22[2213])
#endif
#ifndef XmCLabelSpacing
#define XmCLabelSpacing ((char*)&_XmStrings22[2225])
#endif
#ifndef XmCShowLabel
#define XmCShowLabel ((char*)&_XmStrings22[2238])
#endif
#ifndef XmCStretchable
#define XmCStretchable ((char*)&_XmStrings22[2248])
#endif
#ifndef XmNdefaultEntryLabelAlignment
#define XmNdefaultEntryLabelAlignment ((char*)&_XmStrings22[2260])
#endif
#ifndef XmNdefaultEntryLabelFontList
#define XmNdefaultEntryLabelFontList ((char*)&_XmStrings22[2287])
#endif
#ifndef XmNdefaultFillStyle
#define XmNdefaultFillStyle ((char*)&_XmStrings22[2313])
#endif
#ifndef XmNdistribution
#define XmNdistribution ((char*)&_XmStrings22[2330])
#endif
#ifndef XmNentryLabelAlignment
#define XmNentryLabelAlignment ((char*)&_XmStrings22[2343])
#endif
#ifndef XmNentryLabelFontList
#define XmNentryLabelFontList ((char*)&_XmStrings22[2363])
#endif
#ifndef XmNentryLabelPixmap
#define XmNentryLabelPixmap ((char*)&_XmStrings22[2382])
#endif
#ifndef XmNentryLabelType
#define XmNentryLabelType ((char*)&_XmStrings22[2399])
#endif
#ifndef XmNfillStyle
#define XmNfillStyle ((char*)&_XmStrings22[2414])
#endif
#ifndef XmNitemSpacing
#define XmNitemSpacing ((char*)&_XmStrings22[2424])
#endif
#ifndef XmNlabelSpacing
#define XmNlabelSpacing ((char*)&_XmStrings22[2436])
#endif
#ifndef XmNshowEntryLabel
#define XmNshowEntryLabel ((char*)&_XmStrings22[2449])
#endif
#ifndef XmNstretchable
#define XmNstretchable ((char*)&_XmStrings22[2464])
#endif
#ifndef XmRDistribution
#define XmRDistribution ((char*)&_XmStrings22[2476])
#endif
#ifndef XmRFillStyle
#define XmRFillStyle ((char*)&_XmStrings22[2489])
#endif
#ifndef XmCEqualSize
#define XmCEqualSize ((char*)&_XmStrings22[2499])
#endif
#ifndef XmCFillOption
#define XmCFillOption ((char*)&_XmStrings22[2509])
#endif
#ifndef XmCColorMode
#define XmCColorMode ((char*)&_XmStrings22[2520])
#endif
#ifndef XmCFileReadError
#define XmCFileReadError ((char*)&_XmStrings22[2530])
#endif
#ifndef XmCNoCellError
#define XmCNoCellError ((char*)&_XmStrings22[2544])
#endif
#ifndef XmCSliderLabel
#define XmCSliderLabel ((char*)&_XmStrings22[2556])
#endif
#ifndef XmCTogLabel
#define XmCTogLabel ((char*)&_XmStrings22[2568])
#endif
#ifndef XmCAutoFill
#define XmCAutoFill ((char*)&_XmStrings22[2577])
#endif
#ifndef XmCPicture
#define XmCPicture ((char*)&_XmStrings22[2586])
#endif
#ifndef XmNautoFill
#define XmNautoFill ((char*)&_XmStrings22[2594])
#endif
#ifndef XmNpicture
#define XmNpicture ((char*)&_XmStrings22[2603])
#endif
#ifndef XmNpictureErrorCallback
#define XmNpictureErrorCallback ((char*)&_XmStrings22[2611])
#endif
#ifndef XmNvalidateCallback
#define XmNvalidateCallback ((char*)&_XmStrings22[2632])
#endif
#ifndef XmCFindLabel
#define XmCFindLabel ((char*)&_XmStrings22[2649])
#endif
#ifndef XmCShowFind
#define XmCShowFind ((char*)&_XmStrings22[2659])
#endif
#ifndef XmNentryBackground
#define XmNentryBackground ((char*)&_XmStrings22[2668])
#endif
#ifndef XmCColumnTitles
#define XmCColumnTitles ((char*)&_XmStrings22[2684])
#endif
#ifndef XmCEntryData
#define XmCEntryData ((char*)&_XmStrings22[2697])
#endif
#ifndef XmCFirstColumnPixmaps
#define XmCFirstColumnPixmaps ((char*)&_XmStrings22[2707])
#endif
#ifndef XmCFirstLocation
#define XmCFirstLocation ((char*)&_XmStrings22[2726])
#endif
#ifndef XmCNewVisualStyle
#define XmCNewVisualStyle ((char*)&_XmStrings22[2740])
#endif
#ifndef XmCNumRows
#define XmCNumRows ((char*)&_XmStrings22[2755])
#endif
#ifndef XmCScrollBar
#define XmCScrollBar ((char*)&_XmStrings22[2763])
#endif
#ifndef XmCSelectedColumn
#define XmCSelectedColumn ((char*)&_XmStrings22[2773])
#endif
#ifndef XmNnewVisualStyle
#define XmNnewVisualStyle ((char*)&_XmStrings22[2788])
#endif
#ifndef XmCPreferredPaneSize
#define XmCPreferredPaneSize ((char*)&_XmStrings22[2803])
#endif
#ifndef XmCShowSash
#define XmCShowSash ((char*)&_XmStrings22[2821])
#endif
#ifndef XmC100DPIString
#define XmC100DPIString ((char*)&_XmStrings22[2830])
#endif
#ifndef XmC75DPIString
#define XmC75DPIString ((char*)&_XmStrings22[2843])
#endif
#ifndef XmCAnyLowerString
#define XmCAnyLowerString ((char*)&_XmStrings22[2855])
#endif
#ifndef XmCAnyString
#define XmCAnyString ((char*)&_XmStrings22[2870])
#endif
#ifndef XmCBoldString
#define XmCBoldString ((char*)&_XmStrings22[2880])
#endif
#ifndef XmCBothString
#define XmCBothString ((char*)&_XmStrings22[2891])
#endif
#ifndef XmCDefaultEncodingString
#define XmCDefaultEncodingString ((char*)&_XmStrings22[2902])
#endif
#ifndef XmCEncodingList
#define XmCEncodingList ((char*)&_XmStrings22[2924])
#endif
#ifndef XmCEncodingString
#define XmCEncodingString ((char*)&_XmStrings22[2937])
#endif
#ifndef XmCFamilyString
#define XmCFamilyString ((char*)&_XmStrings22[2952])
#endif
#ifndef XmCItalicString
#define XmCItalicString ((char*)&_XmStrings22[2965])
#endif
#ifndef XmCMonoSpaceString
#define XmCMonoSpaceString ((char*)&_XmStrings22[2978])
#endif
#ifndef XmCOptionString
#define XmCOptionString ((char*)&_XmStrings22[2994])
#endif
#ifndef XmCOtherString
#define XmCOtherString ((char*)&_XmStrings22[3007])
#endif
#ifndef XmCPropSpaceString
#define XmCPropSpaceString ((char*)&_XmStrings22[3019])
#endif
#ifndef XmCSampleText
#define XmCSampleText ((char*)&_XmStrings22[3035])
#endif
#ifndef XmCScalingString
#define XmCScalingString ((char*)&_XmStrings22[3046])
#endif
#ifndef XmCShowNameString
#define XmCShowNameString ((char*)&_XmStrings22[3060])
#endif
#ifndef XmCSizeString
#define XmCSizeString ((char*)&_XmStrings22[3075])
#endif
#ifndef XmCTextRows
#define XmCTextRows ((char*)&_XmStrings22[3086])
#endif
#ifndef XmCXlfdSpaceString
#define XmCXlfdSpaceString ((char*)&_XmStrings22[3095])
#endif
#ifndef XmN100DPIstring
#define XmN100DPIstring ((char*)&_XmStrings22[3111])
#endif
#ifndef XmN75DPIstring
#define XmN75DPIstring ((char*)&_XmStrings22[3124])
#endif
#ifndef XmNanyLowerString
#define XmNanyLowerString ((char*)&_XmStrings22[3136])
#endif
#ifndef XmNanyString
#define XmNanyString ((char*)&_XmStrings22[3151])
#endif
#ifndef XmCCellX
#define XmCCellX ((char*)&_XmStrings22[3161])
#endif
#ifndef XmCCellY
#define XmCCellY ((char*)&_XmStrings22[3167])
#endif
#ifndef XmCDefaultCells
#define XmCDefaultCells ((char*)&_XmStrings22[3173])
#endif
#ifndef XmCMinimumCellSize
#define XmCMinimumCellSize ((char*)&_XmStrings22[3186])
#endif
#ifndef XmCIconPlacement
#define XmCIconPlacement ((char*)&_XmStrings22[3202])
#endif
#ifndef XmNpixmapDepth
#define XmNpixmapDepth ((char*)&_XmStrings22[3216])
#endif
#ifndef XmCAutoClose
#define XmCAutoClose ((char*)&_XmStrings22[3228])
#endif
#ifndef XmCInsertBefore
#define XmCInsertBefore ((char*)&_XmStrings22[3238])
#endif
#ifndef XmCNodeState
#define XmCNodeState ((char*)&_XmStrings22[3251])
#endif
#ifndef XmCNodeStateCallback
#define XmCNodeStateCallback ((char*)&_XmStrings22[3261])
#endif
#ifndef XmCNodeStateChangedCallback
#define XmCNodeStateChangedCallback ((char*)&_XmStrings22[3279])
#endif
#ifndef XmCParentNode
#define XmCParentNode ((char*)&_XmStrings22[3304])
#endif
#ifndef XmNautoClose
#define XmNautoClose ((char*)&_XmStrings22[3315])
#endif
#ifndef XmNcloseFolderPixmap
#define XmNcloseFolderPixmap ((char*)&_XmStrings22[3325])
#endif
#ifndef XmNnodeCloseFolderPixmap
#define XmNnodeCloseFolderPixmap ((char*)&_XmStrings22[3343])
#endif
#ifndef XmNnodeOpenFolderPixmap
#define XmNnodeOpenFolderPixmap ((char*)&_XmStrings22[3365])
#endif
#ifndef XmNnodeStateBeginEndCallback
#define XmNnodeStateBeginEndCallback ((char*)&_XmStrings22[3386])
#endif
#ifndef XmCConstrainWidth
#define XmCConstrainWidth ((char*)&_XmStrings22[3412])
#endif
#ifndef XmCIndentSpace
#define XmCIndentSpace ((char*)&_XmStrings22[3427])
#endif
#ifndef XmNconnectNodes
#define XmNconnectNodes ((char*)&_XmStrings22[3439])
#endif
#ifndef XmNconstrainWidth
#define XmNconstrainWidth ((char*)&_XmStrings22[3452])
#endif
#ifndef XmCNumStacks
#define XmCNumStacks ((char*)&_XmStrings22[3467])
#endif
#ifndef XmCSelectedIndex
#define XmCSelectedIndex ((char*)&_XmStrings22[3477])
#endif
#ifndef XmCStackedEffect
#define XmCStackedEffect ((char*)&_XmStrings22[3491])
#endif
#ifndef XmCTabAutoSelect
#define XmCTabAutoSelect ((char*)&_XmStrings22[3505])
#endif
#ifndef XmCTabCornerPercent
#define XmCTabCornerPercent ((char*)&_XmStrings22[3519])
#endif
#ifndef XmCTabEdge
#define XmCTabEdge ((char*)&_XmStrings22[3536])
#endif
#ifndef XmCTabLabelSpacing
#define XmCTabLabelSpacing ((char*)&_XmStrings22[3544])
#endif
#ifndef XmCTabMode
#define XmCTabMode ((char*)&_XmStrings22[3560])
#endif
#ifndef XmCTabOffset
#define XmCTabOffset ((char*)&_XmStrings22[3568])
#endif
#ifndef XmCTabOrientation
#define XmCTabOrientation ((char*)&_XmStrings22[3578])
#endif
#ifndef XmCTabSelectColor
#define XmCTabSelectColor ((char*)&_XmStrings22[3593])
#endif
#ifndef XmCTabSelectPixmap
#define XmCTabSelectPixmap ((char*)&_XmStrings22[3608])
#endif
#ifndef XmCTabStyle
#define XmCTabStyle ((char*)&_XmStrings22[3624])
#endif
#ifndef XmCTraversalIndex
#define XmCTraversalIndex ((char*)&_XmStrings22[3633])
#endif
#ifndef XmCUniformTabSize
#define XmCUniformTabSize ((char*)&_XmStrings22[3648])
#endif
#ifndef XmCUseImageCache
#define XmCUseImageCache ((char*)&_XmStrings22[3663])
#endif
#ifndef XmCFreeTabPixmap
#define XmCFreeTabPixmap ((char*)&_XmStrings22[3677])
#endif
#ifndef XmCICSEnhancementPakResourceError
#define XmCICSEnhancementPakResourceError ((char*)&_XmStrings22[3691])
#endif
#ifndef XmCTabLabelPixmap
#define XmCTabLabelPixmap ((char*)&_XmStrings22[3722])
#endif
#ifndef XmCTabLabelString
#define XmCTabLabelString ((char*)&_XmStrings22[3737])
#endif
#ifndef XmCTabPixmapPlacement
#define XmCTabPixmapPlacement ((char*)&_XmStrings22[3752])
#endif
#ifndef XmCTabSide
#define XmCTabSide ((char*)&_XmStrings22[3771])
#endif
#ifndef XmNillegalResourceValue
#define XmNillegalResourceValue ((char*)&_XmStrings22[3779])
#endif
#ifndef XmCCompressStyle
#define XmCCompressStyle ((char*)&_XmStrings22[3800])
#endif
#ifndef XmCConnectStyle
#define XmCConnectStyle ((char*)&_XmStrings22[3814])
#endif
#ifndef XmCHorizontalDelta
#define XmCHorizontalDelta ((char*)&_XmStrings22[3827])
#endif
#ifndef XmCLineWidth
#define XmCLineWidth ((char*)&_XmStrings22[3843])
#endif
#ifndef XmCOpenClosePadding
#define XmCOpenClosePadding ((char*)&_XmStrings22[3853])
#endif
#ifndef XmCVerticalDelta
#define XmCVerticalDelta ((char*)&_XmStrings22[3870])
#endif
#ifndef XmNcompressStyle
#define XmNcompressStyle ((char*)&_XmStrings22[3884])
#endif
#ifndef XmNhorizontalDelta
#define XmNhorizontalDelta ((char*)&_XmStrings22[3898])
#endif
#ifndef XmNlineBackgroundColor
#define XmNlineBackgroundColor ((char*)&_XmStrings22[3914])
#endif
#ifndef XmNlineStyle
#define XmNlineStyle ((char*)&_XmStrings22[3934])
#endif
#ifndef XmNverticalDelta
#define XmNverticalDelta ((char*)&_XmStrings22[3944])
#endif
#ifndef XmRXmCompressStyle
#define XmRXmCompressStyle ((char*)&_XmStrings22[3958])
#endif
#ifndef XmRXmLineStyle
#define XmRXmLineStyle ((char*)&_XmStrings22[3974])
#endif
#ifndef XmCPopupOffset
#define XmCPopupOffset ((char*)&_XmStrings22[3986])
#endif
#ifndef XmCUseTextField
#define XmCUseTextField ((char*)&_XmStrings22[3998])
#endif
#ifndef XmCVerify
#define XmCVerify ((char*)&_XmStrings22[4011])
#endif
#endif /* XMSTRINGDEFINES */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* _XmStrDefs22_h_ */
